console_tables = ["delayed_job_failures",
                  "delayed_jobs",
                  "node_classes",
                  "node_group_edges",
                  "node_groups",
                  "nodes",
                  "node_class_memberships",
                  "node_group_class_memberships",
                  "node_group_memberships",
                  "reports",
                  "metrics",
                  "parameters",
                  "report_logs",
                  "resource_statuses",
                  "resource_events",
                  "timeline_events"]

auth_tables = ["activations",
               "users",
               "authorized_users",
               "roles",
               "authorized_users_roles"]

# This comes from dashboard, so we assume we're being called with dashboard on
# the load path
require 'progress_bar'

def row_to_tuple(row)
  "(%s)" % row.map {|v| ActiveRecord::Base.sanitize(v)}.join(',')
end

def export_tables(dir, tables)
  mkdir_p(dir)

  conn = ActiveRecord::Base.connection
  tables.each do |table|
    count = conn.select_one("SELECT COUNT(*) AS count FROM #{table}")['count'].to_i

    pbar = ProgressBar.new(table, count, STDOUT)

    File.open(File.join(dir, "#{table}.json"), 'w') do |f|
      result = conn.execute("SELECT * FROM #{table}")

      fields = result.fetch_fields.map(&:name)
      f.puts fields.to_json

      result.each do |row|
        f.puts row.to_json
        pbar.inc
      end
    end

    pbar.finish
  end
end

def import_tables(dir, tables)
  conn = ActiveRecord::Base.connection
  tables.each do |table|
    file = File.join(dir, "#{table}.json")

    # yay.
    count = %x{wc -l "#{file}"}.split.first.to_i

    pbar = ProgressBar.new(table, count, STDOUT)

    ActiveRecord::Base.transaction do
      File.open(file, 'r') do |f|
        columns = JSON.load(f.readline)

        line_buffer = []
        buffer_size = 10000

        conn.execute("TRUNCATE #{table} CASCADE")

        f.each_line do |line|
          row = JSON.load(line)
          line_buffer << row

          if line_buffer.length == buffer_size
            stmt = "INSERT INTO #{table} (#{columns.join(',')}) VALUES #{line_buffer.map {|row| row_to_tuple(row)}.join(',')}"

            conn.execute(stmt)

            line_buffer.clear
          end

          pbar.inc
        end

        # If we had anything left in the buffer, we have to make sure to write it
        if ! line_buffer.empty?
          stmt = "INSERT INTO #{table} (#{columns.join(',')}) VALUES #{line_buffer.map {|row| row_to_tuple(row)}.join(',')}"

          conn.execute(stmt)
        end
      end

      pbar.finish

      # Inserting rows with specific ids causes the sequence associated with
      # the id column to get out of sync. To fix that, we figure out the
      # sequence (there's only one per table, and it must be the id column) and
      # set it to the highest id in the table.
      seq_results = conn.execute <<-SEQ_SELECT
      SELECT seq.relname
      FROM pg_class AS seq, pg_depend AS dep, pg_class AS tbl, pg_attribute AS c
      WHERE seq.relkind = 'S' AND seq.oid = dep.objid AND dep.refobjid = tbl.oid AND dep.refobjid = c.attrelid AND dep.refobjsubid = c.attnum
      AND tbl.relname = '#{table}'
      SEQ_SELECT

      if result = seq_results.first
        seq = result['relname']
        # 'true' here means we set the current value rather than the next value, so
        # the next value will be one greater than the current max.
        conn.execute("SELECT setval('#{seq}', MAX(id), true) FROM #{table}")
      end
    end
  end
end

namespace :db do
  namespace :console do
    desc 'Export the console database to JSON files in DIR'
    task :export => :environment do
      dir = ENV['DIR']

      export_tables(dir, console_tables)
    end

    desc 'Import the console database from JSON files in DIR'
    task :import => :environment do
      dir = ENV['DIR']

      import_tables(dir, console_tables)
    end
  end

  namespace :console_auth do
    desc 'Export the console_auth database to JSON files in DIR'
    task :export => 'db:connect' do
      dir = ENV['DIR']

      export_tables(dir, auth_tables)
    end

    desc 'Import the console_auth database from JSON files in DIR'
    task :import => 'db:connect' do
      dir = ENV['DIR']

      import_tables(dir, auth_tables)
    end
  end
end
